import{a as j,t as v}from"./BJvdQk2J.js";import{r as I}from"./ByuL26re.js";import{w as f}from"./BQX52OTj.js";function z(e){return(e==null?void 0:e.length)!==void 0?e:Array.from(e)}function M(e,d){v(e,1,1,()=>{d.delete(e.key)})}function C(e,d){e.f(),M(e,d)}function G(e,d,r,_,w,u,s,l,n,k,g,L){let c=e.length,h=u.length,i=c;const y={};for(;i--;)y[e[i].key]=i;const m=[],A=new Map,T=new Map,E=[];for(i=h;i--;){const t=L(w,u,i),o=r(t);let a=s.get(o);a?E.push(()=>a.p(t,d)):(a=k(o,t),a.c()),A.set(o,m[i]=a),o in y&&T.set(o,Math.abs(i-y[o]))}const D=new Set,b=new Set;function p(t){j(t,1),t.m(l,g),s.set(t.key,t),g=t.first,h--}for(;c&&h;){const t=m[h-1],o=e[c-1],a=t.key,S=o.key;t===o?(g=t.first,c--,h--):A.has(S)?!s.has(a)||D.has(a)?p(t):b.has(S)?c--:T.get(a)>T.get(S)?(b.add(a),p(t)):(D.add(S),c--):(n(o,s),c--)}for(;c--;){const t=e[c];A.has(t.key)||n(t,s)}for(;h;)p(m[h-1]);return I(E),m}function N(e,d){const r={},_={},w={$$scope:1};let u=e.length;for(;u--;){const s=e[u],l=d[u];if(l){for(const n in s)n in l||(_[n]=1);for(const n in l)w[n]||(r[n]=l[n],w[n]=1);e[u]=l}else for(const n in s)w[n]=1}for(const s in _)s in r||(r[s]=void 0);return r}function V(e){return typeof e=="object"&&e!==null?e:{}}const x=20,B=1,F={currentPage:B,pageSize:x},Z=f(),q=f(),H=f(),J=f([]),K=f([]),O=f(!0),Q=f(F),R=f({shouldSave:!1,canSave:!1});export{x as D,V as a,R as b,Z as c,H as d,z as e,Q as f,N as g,F as h,O as i,C as j,B as k,q as l,J as m,M as o,K as s,G as u};
